@@ -5,7 +5,7 @@ use strict;
 use warnings;
 
 use List::Util      qw/any min/;
-use List::MoreUtils qw/zip first_index/;
+use List::MoreUtils qw/zip first_index true/;
 use Scalar::Util    qw/looks_like_number/;
 
 =head1 NAME
@@ -14,11 +14,11 @@ Games::Cards::ShuffleTrack - Track cards through shuffles and cuts
 
 =head1 VERSION
 
-Version 0.04
+Version 0.04_1
 
 =cut
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 my $cut_limits = {
 	normal  => [0.19, 0.82], # on a 52 cards deck, cut between 10 and 43 cards
@@ -236,6 +236,44 @@ sub original_size {
 }
 
 
+=head3 is_original
+
+Checks to see if the order of the deck is still the same as the original order.
+
+    if ($deck->is_original) {
+        # ...
+    }
+
+This method checks for each card and also the orientation of the deck. The only case where the orientation of the deck doesn't matter is when the deck is empty, in which case that property is ignored.
+
+=cut
+
+sub is_original {
+    my $self = shift;
+
+    # check size
+    my @original = @{$self->{'original'}};
+    if ( $self->size != @original ) {
+        return 0;
+    }
+
+    $self->size || return 1;
+
+    # check orientation
+    if ( $self->orientation ne 'down' ) {
+        return 0;
+    }
+    
+    # check order
+    my @deck = @{$self->get_deck};
+    for ( 0 .. $#deck ) {
+        $deck[$_] eq $original[$_] || return 0;
+    }
+
+    return 1;
+}
+
+
 =head3 get_deck
 
 Returns the deck (a reference to a list of strings).
@@ -321,8 +359,8 @@ Since you can add whichever card you want to the deck, it should be noted how se
 It is important to note:
 
 	my $total = $deck->count( 'JC' );   # holds 4
-	my $total = $deck->count( 'C', J ); # holds 16, because the JC is only counted once
-	my @total = $deck->count( 'C', J ); # holds (13, 4)
+	my $total = $deck->count( 'C', 'J' ); # holds 16, because the JC is only counted once
+	my @total = $deck->count( 'C', 'J' ); # holds (13, 4)
 
 Also:
 
@@ -352,10 +390,10 @@ sub _count_each {
 	my @results;
 	while (my $param = shift) {
 		if ( exists $expressions->{$param} ) {
-			push @results, scalar grep { /$expressions->{$param}/ } @{$self->get_deck};
+			push @results, true { /$expressions->{$param}/ } @{$self->get_deck};
 		}
 		else {
-			push @results, scalar grep { /$param/ } @{$self->get_deck};
+			push @results, true { /$param/ } @{$self->get_deck};
 		}
 	}
 	return @results;
@@ -1112,19 +1150,20 @@ sub deal {
 		return $self;
 	}
 
-	my $deal        = 'top';
-	my $destination = undef;
-	while (my $param = shift) {
-		if ( ref( $param ) eq 'Games::Cards::ShuffleTrack' ) {
-			$destination = $param;
-		}
-		elsif ( exists $shortcuts->{$param} ) {
-			$deal = $param;
+	my $params = _parse_params(@_);
+
+	my $destination = $params->{_has_places} ?
+						$params->{'places'}[0] :
+						undef;
+
+	my $position = $shortcuts->{'top'};
+	if ($params->{_has_options}) {
+		my $param = $params->{'options'}->[0];
+		if (exists $shortcuts->{$param}) {
+			$position = $shortcuts->{$param};
 		}
 	}
 
-	my $position = $shortcuts->{$deal};
-
 	my $card = $self->remove( $position );
 
 	if ( defined $destination ) {
@@ -1219,6 +1258,43 @@ sub take_random {
 	return $self->remove( _rand( $lower_limit, $upper_limit ) );
 }
 
+
+=head3 remove_all
+
+Removes all cards that match a pattern from the deck.
+
+    $deck->remove_all( 'Joker' ); # remove all Jokers
+    $deck->remove_all( 'A' ); # remove all Aces
+    $deck->remove_all( 'C' ); # remove all Clubs
+    $deck->remove_all( 'J', 'Q', 'K' ); # remove all court cards
+
+Without arguments this method does precisely what it states:
+
+    $deck->remove_all(); # removes everythin from the deck
+
+=cut
+
+sub remove_all {
+    my $self = shift;
+
+    if ( @_ ) {
+        while (my $param = shift) {
+            if ( exists $expressions->{$param} ) {
+                $self->_set_deck( grep { not /$expressions->{$param}/ } @{$self->get_deck} );
+            }
+            else {
+                $self->_set_deck( grep { not /$param/ } @{$self->get_deck} );
+            }
+        }
+    }
+    else {
+        $self->_set_deck();
+    }
+    
+    return $self;
+}
+
+
 =head3 dribble
 
 Dribble the cards (usually to select one, which could either be the last one to fall or the one that would be next).
@@ -1262,9 +1338,8 @@ sub dribble {
 						$lower_limit;
 	}
 	else {
-		# FIXME: this means we can't dribble if we have just 5 cards
-		$lower_limit = 5; # minimum amount of cards to hold back
-		$upper_limit = $self->size - 5; # minimum amount of cards to dribble
+		$lower_limit = min( $self->size, 5 );
+		$upper_limit = $self->size < 5 ? $self->size : $self->size - 5;
 	}
 
 	$self->turn;
@@ -1307,6 +1382,10 @@ sub _parse_params {
 			push @{$params->{'options'}}, $param;
 		}
 	}
+
+	for (qw/numbers places options/) {
+		$params->{"_has_$_"} = @{$params->{$_}};
+	}
 	return $params;
 }
 
